* This advanced example shows binding to task dialog events, including clicking
* hyperlinks, and adding additional controls, such as a progress bar and
* command link. It would be nice to have a single dialog that does both tasks
* but clicking a command link closes the dialog, so we need a second one.

#include VistaDialogs.H
local loTaskDialog, ;
	loTaskDialogEventHandler, ;
	loOurEventHandler, ;
	loButton, ;
	loCommandLinkEventHandler, ;
	loTaskDialogResult, ;
	lcResult

* Instantiate the task dialog object and set some properties.

loTaskDialog = createobject('VistaDialogs4COM.TaskDialog')
with loTaskDialog
	.Caption         = 'My Application'
	.Instruction     = 'A new version is available.'
	.Content         = 'For information on the features in this new ' + ;
		'release, see <a href="http://www.stonefieldquery.com">our Web ' + ;
		'site</a>.'
	.MainIcon        = TDINFORMATIONICON
	.StandardButtons = TDCLOSE
	.Cancelable      = .T.

* Create an event handler to deal with events raised by the dialog.

	loTaskDialogEventHandler = newobject('TaskDialogEventHandler', ;
		'DialogEventHandlers.PRG')
	eventhandler(loTaskDialog, loTaskDialogEventHandler)
	loOurEventHandler = createobject('DialogEventHandler')

* When the user clicks a hyperlink, an event fires allowing you to determine
* what to do. Let's bind to that event.

	.HyperlinksEnabled = .T.
	bindevent(loTaskDialogEventHandler, 'ITaskDialogEvents_TDHyperlinkClick', ;
		loOurEventHandler, 'OnHyperlinkClicked')

* Create a command link button and bind to its click event.

	loButton = createobject('VistaDialogs4COM.TaskDialogCommandLink')
*	loButton.Text = 'This will download the new version to your system ' + ;
		'and overwrite the current version.'
		&& this property is currently implemented as Logical but it should be
		&& a string
	loButton.Instruction = 'Download new version'
	loTaskDialog.AddControl(loButton)
	loCommandLinkEventHandler = newobject('TaskDialogCommandLinkEventHandler', ;
		'DialogEventHandlers.PRG')
	eventhandler(loButton, loCommandLinkEventHandler)
	bindevent(loCommandLinkEventHandler, 'ITaskDialogCommandLinkEvents_TDClick', ;
		loOurEventHandler, 'OnCommandLinkClicked')

* Display the dialog.

	.Show()

* Unbind the event handler.

	eventhandler(loTaskDialog, loTaskDialogEventHandler, .T.)
endwith

* If the user clicked the command link, show a progress dialog.

if loOurEventHandler.lCommandLinkClicked
	loTaskDialog = createobject('VistaDialogs4COM.TaskDialog')
	with loTaskDialog
		.Caption         = 'My Application'
		.Instruction     = 'Downloading new version'
		.Content         = 'To cancel the download, click the Cancel button'
		.MainIcon        = TDINFORMATIONICON
		.StandardButtons = TDCANCEL
		.Cancelable      = .T.

* Handle events raised by the dialog.

		eventhandler(loTaskDialog, loTaskDialogEventHandler)

* Let's show a progress bar and bind to the tick event of the dialog.

		loOurEventHandler.oProgressBar = createobject('VistaDialogs4COM.TaskDialogProgressBar')
		loTaskDialog.AddControl(loOurEventHandler.oProgressBar)
		bindevent(loTaskDialogEventHandler, 'ITaskDialogEvents_TDTick', ;
			loOurEventHandler, 'OnTick')

* To show a marquee instead, do this:

*		loMarquee = createobject('VistaDialogs4COM.TaskDialogMarquee')
*		loTaskDialog.AddControl(loMarquee)

* Display the dialog and determine if the user pressed cancel or not.

		loTaskDialogResult = .Show()
		lcResult           = loTaskDialogResult.ToString()
		if 'cancelled: True' $ lcResult
			messagebox('You cancelled the download.')
		endif 'cancelled: True' $ lcResult
	endwith
endif loOurEventHandler.lCommandLinkClicked

* Clean up before we exit.

unbindevents(loTaskDialogEventHandler)
unbindevents(loCommandLinkEventHandler)


* Event handler class.

define class DialogEventHandler as Custom
	oProgressBar = .NULL.
		&& a reference to a VistaDialogs4COM.TaskDialogProgressBar object
	lCommandLinkClicked = .F.
		&& .T. if the user clicked the command link
	lDoneProcess = .F.
		&& .T. if the process is complete

* When the user clicks a hyperlink, navigate to it.

	function OnHyperlinkClicked(tcLinkText)
		declare integer ShellExecute in SHELL32.DLL ;
			integer nWinHandle, string cOperation, string cFileName, ;
			string cParameters, string cDirectory, integer nShowWindow
		ShellExecute(0, '', tcLinkText, '', '', 0)
	endfunc

* Set a flag when the user clicks the command link.

	function OnCommandLinkClicked
		This.lCommandLinkClicked = .T.
	endfunc

* This doesn't do much: it only advances the progress bar. A real application
* would do some work here.

	function OnTick(tnTicks)
		if not This.lDoneProcess
			This.oProgressBar.Value = This.oProgressBar.Value + 5
			if This.oProgressBar.Value >= 100
				This.lDoneProcess = .T.
*** There isn't a programmatic way to close the dialog at this point.
			endif This.oProgressBar.Value >= 100
		endif not This.lDoneProcess
	endfunc
enddefine
