*
*  WEEKDAYS.PRG
*  Return the number of days that are MONDAY - FRIDAY
*  in the passed date range WITHOUT LOOPING!
*
*  Author:  Mike Yearwood with input from various
*                UniversalThread and Foxite members.
*
*  USAGE:
*    ?WEEKDAYS({^2004-05-01},{^2004-05-31})
*
*  OR:
*    ldFrom = {^2004-05-01}
*    ldTo = {^2004-05-31}
*    lnWeekdays = WEEKDAYS(m.ldFrom, m.ldTo)
*    ?"The number of weekdays was: " + ALLTRIM(STR(m.lnWeekdays)) + "."
*
*  RETURNS:
*    A numeric value for the number of weekdays.
*    This also indicates the routine succeeded.
*    It crashes when called with incorrect parameters.
*
*  lParameters
*     tdFrom (R)     Start of the date range.
*     tdTo   (R)     End of the date range.
*
Lparameters m.tuFrom, m.tuTo

Local ;
	ldFrom, ldTo, lnDays, lnDOW

*Martina Jindrova suggested converting DateTimes to Dates.
*I use locals rather than the m.tuFrom and m.tuTo
*because if they were passed by reference, this function
*has no business altering the passed values.

*Part 1 of the formula subtracts Saturday and Sunday from
*the end of the week(s).

**** m.lnDays - (INT(m.lnDays / 7) * 2)

*The rest of the formula drops any leading
*Saturday or Sunday from any partial week.

*We could use IIF() but that just makes
*the formula harder to read/write
*because (m.lnDays % 7) is repeated as is
*SIGN(). Both are calculated every time.
*SIGN() returns 1/0 instead of T/F.

*It seems the formula can be shortened
*without much impact on performance.

*There are three parts to consider.

*Part1 determines if there is a
*partial week.
**** SIGN( (m.lnDays % 7)

*Part2 is looking to drop a leading
*Saturday from that partial week.
**** + m.lnDOW - 7

*Part3 is looking to drop a leading
*Sunday from that partial week.
**** SIGN(m.lnDOW % 7)

*Here's the result table where there are no
*remaining days. Part 1 is 0.
*Day  Part1    Part2   Part3         1+2  3   1+2+3
*Mon -(sign(0 + -6) + sign(1%7)) = -(-1 + 1) =  0
*Tue -(sign(0 + -5) + sign(2%7)) = -(-1 + 1) =  0
*Sat -(sign(0 + -1) + sign(6%7)) = -(-1 + 1) =  0
*Sun -(sign(0 +  0) + sign(7%7)) = -( 0 + 0) =  0

*If there are remaining days, Part 1 is 1.
*Part2 and Part3 trigger on Saturday or Sunday.
*Day  Part1    Part2   Part3         1+2  3   1+2+3
*Mon -(sign(1 + -6) + sign(1%7)) = -(-1 + 1) =  0
*Tue -(sign(1 + -5) + sign(2%7)) = -(-1 + 1) =  0
*Sat -(sign(1 + -1) + sign(6%7)) = -( 0 + 1) = -1 && drop Sat
*Sun -(sign(1 +  0) + sign(7%7)) = -( 1 + 0) = -1 && drop Sun

*Original deprecated formula.
*!*    lnReturn = m.lnDays - (INT(m.lnDays / 7) * 2) ;
*!*        - IIF( m.lnDOW = 7, ;
*!*             SIGN(m.lnDays % 7), ;
*!*             SIGN(m.lnDOW - 7 + (m.lnDays % 7)) + 1)

ldFrom = Iif(Vartype(m.tuFrom)="T",Ttod(m.tuFrom),m.tuFrom)
ldTo = Iif(Vartype(m.tuTo)="T",Ttod(m.tuTo),m.tuTo)

*Determine the total possible number of days.
lnDays = (m.ldTo - m.ldFrom) + 1

lnDOW = Dow(m.ldFrom,2)

Return m.lnDays - (Int(m.lnDays / 7) * 2) ;
	- (Sign( (m.lnDays % 7) + m.lnDOW - 7) + Sign(m.lnDOW % 7))
